/****************************************************************************
 * Copyright (C) 2016 Intel Corporation.   All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 * @file gen_ar_eventhandlerfile.hpp
 *
 * @brief Event handler interface.  auto-generated file
 *
 * DO NOT EDIT
 *
 * Generation Command Line:
 *  ../src/gallium/drivers/swr/rasterizer/codegen/gen_archrast.py
 *    --proto
 *    ../src/gallium/drivers/swr/rasterizer/archrast/events.proto
 *    ../src/gallium/drivers/swr/rasterizer/archrast/events_private.proto
 *    --output-dir
 *    /home/iurt/rpmbuild/BUILD/mesa-21.1.2/build/src/gallium/drivers/swr/rasterizer/codegen
 *
 ******************************************************************************/
// clang-format off
#pragma once

#include "common/os.h"
#include "gen_ar_eventhandler.hpp"
#include <fstream>
#include <sstream>
#include <iostream>
#include <thread>

namespace ArchRast
{
    //////////////////////////////////////////////////////////////////////////
    /// EventHandlerFile - interface for handling events.
    //////////////////////////////////////////////////////////////////////////
    class EventHandlerFile : public EventHandler
    {
    public:
        EventHandlerFile(uint32_t id) : mBufOffset(0)
        {
#if defined(_WIN32)
            DWORD pid = GetCurrentProcessId();
            TCHAR procname[MAX_PATH];
            GetModuleFileName(NULL, procname, MAX_PATH);
            const char*       pBaseName = strrchr(procname, '\\');
            std::stringstream outDir;
            outDir << KNOB_DEBUG_OUTPUT_DIR << pBaseName << "_" << pid << std::ends;
            mOutputDir = outDir.str();
            if (CreateDirectory(mOutputDir.c_str(), NULL))
            {
                std::cout << std::endl
                          << "ArchRast Dir:       " << mOutputDir << std::endl
                          << std::endl
                          << std::flush;
            }

            // There could be multiple threads creating thread pools. We
            // want to make sure they are uniquely identified by adding in
            // the creator's thread id into the filename.
            std::stringstream fstr;
            fstr << outDir.str().c_str() << "\\ar_event" << std::this_thread::get_id();
            fstr << "_" << id << ".bin" << std::ends;
            mFilename = fstr.str();
#else
            // There could be multiple threads creating thread pools. We
            // want to make sure they are uniquely identified by adding in
            // the creator's thread id into the filename.
            std::stringstream fstr;
            fstr << "/tmp/ar_event" << std::this_thread::get_id();
            fstr << "_" << id << ".bin" << std::ends;
            mFilename = fstr.str();
#endif
        }

        virtual ~EventHandlerFile() { FlushBuffer(); }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Flush buffer to file.
        bool FlushBuffer()
        {
            if (mBufOffset > 0)
            {
                if (mBufOffset == mHeaderBufOffset)
                {
                    // Nothing to flush. Only header has been generated.
                    return false;
                }

                std::ofstream file;
                file.open(mFilename, std::ios::out | std::ios::app | std::ios::binary);

                if (!file.is_open())
                {
                    SWR_INVALID("ArchRast: Could not open event file!");
                    return false;
                }

                file.write((char*)mBuffer, mBufOffset);
                file.close();

                mBufOffset       = 0;
                mHeaderBufOffset = 0; // Reset header offset so its no longer considered.
            }
            return true;
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Write event and its payload to the memory buffer.
        void Write(uint32_t eventId, const char* pBlock, uint32_t size)
        {
            if ((mBufOffset + size + sizeof(eventId)) > mBufferSize)
            {
                if (!FlushBuffer())
                {
                    // Don't corrupt what's already in the buffer?
                    /// @todo Maybe add corrupt marker to buffer here in case we can open file in
                    /// future?
                    return;
                }
            }

            memcpy(&mBuffer[mBufOffset], (char*)&eventId, sizeof(eventId));
            mBufOffset += sizeof(eventId);
            memcpy(&mBuffer[mBufOffset], pBlock, size);
            mBufOffset += size;
        }


        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle ApiSwr::FrameEndEvent event
        virtual void Handle(const FrameEndEvent& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle ApiSwr::SwrDiscardRectEvent event
        virtual void Handle(const SwrDiscardRectEvent& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle ApiSwr::SwrInvalidateTilesEvent event
        virtual void Handle(const SwrInvalidateTilesEvent& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle ApiSwr::SwrStoreTilesEvent event
        virtual void Handle(const SwrStoreTilesEvent& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle ApiSwr::SwrSyncEvent event
        virtual void Handle(const SwrSyncEvent& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle Framework::ThreadStartApiEvent event
        virtual void Handle(const ThreadStartApiEvent& event)
        {
            Write(event.eventId, (char*)&event.data, 0);
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle Framework::ThreadStartWorkerEvent event
        virtual void Handle(const ThreadStartWorkerEvent& event)
        {
            Write(event.eventId, (char*)&event.data, 0);
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle Memory::MemoryAccessEvent event
        virtual void Handle(const MemoryAccessEvent& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle Memory::MemoryStatsEndEvent event
        virtual void Handle(const MemoryStatsEndEvent& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::AlphaEvent event
        virtual void Handle(const AlphaEvent& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::AlphaInfoEvent event
        virtual void Handle(const AlphaInfoEvent& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::BackendDrawEndEvent event
        virtual void Handle(const BackendDrawEndEvent& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::BackendStatsEvent event
        virtual void Handle(const BackendStatsEvent& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::ClipInfoEvent event
        virtual void Handle(const ClipInfoEvent& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::ClipperEvent event
        virtual void Handle(const ClipperEvent& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::CullEvent event
        virtual void Handle(const CullEvent& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::CullInfoEvent event
        virtual void Handle(const CullInfoEvent& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::DispatchEvent event
        virtual void Handle(const DispatchEvent& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::DrawIndexedInstancedEvent event
        virtual void Handle(const DrawIndexedInstancedEvent& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::DrawInfoEvent event
        virtual void Handle(const DrawInfoEvent& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::DrawInstancedEvent event
        virtual void Handle(const DrawInstancedEvent& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::EarlyDepthInfoPixelRate event
        virtual void Handle(const EarlyDepthInfoPixelRate& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::EarlyDepthStencilInfoNullPS event
        virtual void Handle(const EarlyDepthStencilInfoNullPS& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::EarlyDepthStencilInfoSampleRate event
        virtual void Handle(const EarlyDepthStencilInfoSampleRate& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::EarlyDepthStencilInfoSingleSample event
        virtual void Handle(const EarlyDepthStencilInfoSingleSample& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::EarlyOmStencil event
        virtual void Handle(const EarlyOmStencil& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::EarlyOmZ event
        virtual void Handle(const EarlyOmZ& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::EarlyStencil event
        virtual void Handle(const EarlyStencil& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::EarlyStencilNullPS event
        virtual void Handle(const EarlyStencilNullPS& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::EarlyStencilSampleRate event
        virtual void Handle(const EarlyStencilSampleRate& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::EarlyStencilSingleSample event
        virtual void Handle(const EarlyStencilSingleSample& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::EarlyZ event
        virtual void Handle(const EarlyZ& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::EarlyZNullPS event
        virtual void Handle(const EarlyZNullPS& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::EarlyZPixelRate event
        virtual void Handle(const EarlyZPixelRate& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::EarlyZSampleRate event
        virtual void Handle(const EarlyZSampleRate& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::EarlyZSingleSample event
        virtual void Handle(const EarlyZSingleSample& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::FrontendDrawEndEvent event
        virtual void Handle(const FrontendDrawEndEvent& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::FrontendStatsEvent event
        virtual void Handle(const FrontendStatsEvent& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::GSInputPrims event
        virtual void Handle(const GSInputPrims& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::GSPrimInfo event
        virtual void Handle(const GSPrimInfo& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::GSPrimsGen event
        virtual void Handle(const GSPrimsGen& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::GSVertsInput event
        virtual void Handle(const GSVertsInput& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::LateDepthInfoPixelRate event
        virtual void Handle(const LateDepthInfoPixelRate& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::LateDepthStencilInfoNullPS event
        virtual void Handle(const LateDepthStencilInfoNullPS& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::LateDepthStencilInfoSampleRate event
        virtual void Handle(const LateDepthStencilInfoSampleRate& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::LateDepthStencilInfoSingleSample event
        virtual void Handle(const LateDepthStencilInfoSingleSample& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::LateOmStencil event
        virtual void Handle(const LateOmStencil& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::LateOmZ event
        virtual void Handle(const LateOmZ& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::LateStencil event
        virtual void Handle(const LateStencil& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::LateStencilSampleRate event
        virtual void Handle(const LateStencilSampleRate& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::LateStencilSingleSample event
        virtual void Handle(const LateStencilSingleSample& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::LateZ event
        virtual void Handle(const LateZ& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::LateZPixelRate event
        virtual void Handle(const LateZPixelRate& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::LateZSampleRate event
        virtual void Handle(const LateZSampleRate& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::LateZSingleSample event
        virtual void Handle(const LateZSingleSample& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::RasterTileCount event
        virtual void Handle(const RasterTileCount& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::RasterTiles event
        virtual void Handle(const RasterTiles& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::TessPrimCount event
        virtual void Handle(const TessPrimCount& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle PipelineStats::TessPrims event
        virtual void Handle(const TessPrims& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle ShaderStats::CSInfo event
        virtual void Handle(const CSInfo& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle ShaderStats::CSStats event
        virtual void Handle(const CSStats& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle ShaderStats::DSInfo event
        virtual void Handle(const DSInfo& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle ShaderStats::DSStats event
        virtual void Handle(const DSStats& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle ShaderStats::GSInfo event
        virtual void Handle(const GSInfo& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle ShaderStats::GSStats event
        virtual void Handle(const GSStats& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle ShaderStats::HSInfo event
        virtual void Handle(const HSInfo& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle ShaderStats::HSStats event
        virtual void Handle(const HSStats& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle ShaderStats::PSInfo event
        virtual void Handle(const PSInfo& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle ShaderStats::PSStats event
        virtual void Handle(const PSStats& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle ShaderStats::VSInfo event
        virtual void Handle(const VSInfo& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Handle ShaderStats::VSStats event
        virtual void Handle(const VSStats& event)
        {
            Write(event.eventId, (char*)&event.data, sizeof(event.data));
        }

        //////////////////////////////////////////////////////////////////////////
        /// @brief Everything written to buffer this point is the header.
        virtual void MarkHeader()
        {
            mHeaderBufOffset = mBufOffset;
        }

        std::string mFilename;
        std::string mOutputDir;

        static const uint32_t mBufferSize = 1024;
        uint8_t               mBuffer[mBufferSize];
        uint32_t mBufOffset{0};
        uint32_t mHeaderBufOffset{0};
    };
} // namespace ArchRast
// clang-format on
